@@ -1,9 +1,14 @@
+from __future__ import print_function
+
 try:
     import queue
 except ImportError:
     import Queue as queue
 import Pyro4
 
+Pyro4.config.SERIALIZER='pickle'   # we're using custom classes
+
+
 class DispatcherQueue(object):
     def __init__(self):
         self.workqueue = queue.Queue()
@@ -27,7 +32,6 @@ ns=Pyro4.naming.locateNS()
 daemon=Pyro4.core.Daemon()
 dispatcher=DispatcherQueue()
 uri=daemon.register(dispatcher)
-ns.remove("example.distributed.dispatcher")
 ns.register("example.distributed.dispatcher", uri)
 print("Dispatcher is ready.")
 daemon.requestLoop()
