@@ -1,3 +1,4 @@
+from __future__ import print_function
 import os,socket,sys
 from math import sqrt
 try:
@@ -10,6 +11,8 @@ from workitem import Workitem
 if sys.version_info<(3,0):
     range=xrange
 
+Pyro4.config.SERIALIZER='pickle'   # we're using custom classes
+
 WORKERNAME = "Worker_%d@%s" % (os.getpid(), socket.gethostname())
 
 def factorize(n):
