@@ -21,7 +21,7 @@ class Configuration(object):
                "THREADPOOL_MINTHREADS", "THREADPOOL_MAXTHREADS",
                "THREADPOOL_IDLETIMEOUT", "HMAC_KEY", "AUTOPROXY",
                "BROADCAST_ADDRS", "NATHOST", "NATPORT", "MAX_MESSAGE_SIZE",
-               "FLAME_ENABLED" )
+               "FLAME_ENABLED", "SERIALIZER", "LOGWIRE" )
 
     def __init__(self):
         self.reset()
@@ -56,6 +56,8 @@ class Configuration(object):
         self.BROADCAST_ADDRS = "<broadcast>, 0.0.0.0"   # comma separated list of broadcast addresses
         self.FLAME_ENABLED = False
         self.PREFER_IP_VERSION = 4    # 4, 6 or 0 (let OS choose according to RFC 3484)
+        self.SERIALIZER = "serpent"
+        self.LOGWIRE = False   # log wire-level messages
 
         if useenvironment:
             # process enviroment variables
@@ -78,9 +80,8 @@ class Configuration(object):
                         else:
                             envvalue=valuetype(envvalue)  # just cast the value to the appropriate type
                     setattr(self, symbol, envvalue)
-        if self.HMAC_KEY and sys.version_info>=(3,0):
-            if type(self.HMAC_KEY) is not bytes:
-                self.HMAC_KEY=bytes(self.HMAC_KEY, "utf-8")     # convert to bytes
+        if self.HMAC_KEY and type(self.HMAC_KEY) is not bytes:
+            self.HMAC_KEY = self.HMAC_KEY.encode("utf-8")     # convert to bytes
 
     def asDict(self):
         """returns the current config as a regular dictionary"""
