@@ -8,22 +8,18 @@ from __future__ import with_statement
 import re, struct, sys, time, os
 import logging, uuid
 import hashlib, hmac
-try:
-    import copyreg
-except ImportError:
-    import copy_reg as copyreg
 from Pyro4 import constants, threadutil, util, socketutil, errors
 from Pyro4.socketserver.threadpoolserver import SocketServer_Threadpool
 from Pyro4.socketserver.multiplexserver import SocketServer_Select, SocketServer_Poll
 from Pyro4 import futures
 import Pyro4
 
-__all__=["URI", "Proxy", "Daemon", "callback", "batch", "async"]
+__all__ = ["URI", "Proxy", "Daemon", "callback", "batch", "async"]
 
-if sys.version_info>=(3,0):
-    basestring=str
+if sys.version_info >= (3, 0):
+    basestring = str
 
-log=logging.getLogger("Pyro4.core")
+log = logging.getLogger("Pyro4.core")
 
 
 class URI(object):
@@ -37,8 +33,8 @@ class URI(object):
     There is also a 'Magic format' for simple name resolution using Name server:
       ``PYRONAME:objectname[@location]``  (optional name server location, can also omit location port)
     """
-    uriRegEx=re.compile(r"(?P<protocol>PYRO[A-Z]*):(?P<object>\S+?)(@(?P<location>\S+))?$")
-    __slots__=("protocol", "object", "sockname", "host", "port", "object")
+    uriRegEx = re.compile(r"(?P<protocol>PYRO[A-Z]*):(?P<object>\S+?)(@(?P<location>\S+))?$")
+    __slots__ = ("protocol", "object", "sockname", "host", "port", "object")
 
     def __init__(self, uri):
         if isinstance(uri, URI):
@@ -113,7 +109,7 @@ class URI(object):
 
     def __str__(self):
         string=self.asString()
-        if sys.version_info<(3,0) and type(string) is unicode:
+        if sys.version_info<(3, 0) and type(string) is unicode:
             return string.encode("ascii", "replace")
         return string
 
@@ -127,7 +123,8 @@ class URI(object):
         if not isinstance(other, URI):
             return False
         return (self.protocol, self.object, self.sockname, self.host, self.port) \
-                == (other.protocol, other.object, other.sockname, other.host, other.port)
+            == (other.protocol, other.object, other.sockname, other.host, other.port)
+
     def __ne__(self, other):
         return not self.__eq__(other)
 
@@ -140,6 +137,9 @@ class URI(object):
     def __getstate__(self):
         return self.protocol, self.object, self.sockname, self.host, self.port
 
+    def __getstate_for_dict__(self):
+        return self.__getstate__()
+
     def __setstate__(self, state):
         self.protocol, self.object, self.sockname, self.host, self.port = state
 
@@ -161,7 +161,7 @@ def _check_hmac():
     if Pyro4.config.HMAC_KEY is None or len(Pyro4.config.HMAC_KEY)==0:
         import warnings
         warnings.warn("HMAC_KEY not set, protocol data may not be secure")
-    elif sys.version_info>=(3,0) and type(Pyro4.config.HMAC_KEY) is not bytes:
+    elif sys.version_info>=(3, 0) and type(Pyro4.config.HMAC_KEY) is not bytes:
         raise errors.PyroError("HMAC_KEY must be bytes type")
 
 
@@ -175,7 +175,6 @@ class Proxy(object):
     .. automethod:: _pyroBatch
     .. automethod:: _pyroAsync
     """
-    _pyroSerializer=util.Serializer()
     __pyroAttributes=frozenset(["__getnewargs__", "__getinitargs__", "_pyroConnection", "_pyroUri", "_pyroOneway", "_pyroTimeout", "_pyroSeq"])
 
     def __init__(self, uri):
@@ -192,6 +191,7 @@ class Proxy(object):
         self._pyroConnection=None
         self._pyroOneway=set()
         self._pyroSeq=0    # message sequence number
+        self._pyroSerializer = util.get_serializer()
         self.__pyroTimeout=Pyro4.config.COMMTIMEOUT
         self.__pyroLock=threadutil.Lock()
         self.__pyroConnLock=threadutil.Lock()
@@ -209,18 +209,22 @@ class Proxy(object):
     def __repr__(self):
         connected="connected" if self._pyroConnection else "not connected"
         return "<%s.%s at 0x%x, %s, for %s>" % (self.__class__.__module__, self.__class__.__name__,
-            id(self), connected, self._pyroUri)
+               id(self), connected, self._pyroUri)
 
     def __unicode__(self):
         return str(self)
 
     def __getstate__(self):
-        return self._pyroUri, self._pyroOneway, self._pyroSerializer, self.__pyroTimeout    # skip the connection
+        return self._pyroUri, self._pyroOneway, self.__pyroTimeout    # skip the connection
+
+    def __getstate_for_dict__(self):
+        return self._pyroUri.asString(), tuple(self._pyroOneway), self.__pyroTimeout
 
     def __setstate__(self, state):
-        self._pyroUri, self._pyroOneway, self._pyroSerializer, self.__pyroTimeout = state
+        self._pyroUri, self._pyroOneway, self.__pyroTimeout = state
         self._pyroConnection=None
         self._pyroSeq=0
+        self._pyroSerializer = util.get_serializer()
         self.__pyroLock=threadutil.Lock()
         self.__pyroConnLock=threadutil.Lock()
 
@@ -277,8 +281,8 @@ class Proxy(object):
         if self._pyroConnection is None:
             # rebind here, don't do it from inside the invoke because deadlock will occur
             self.__pyroCreateConnection()
-        data, compressed=self._pyroSerializer.serialize(
-            (self._pyroConnection.objectId, methodname, vargs, kwargs),
+        data, compressed=self._pyroSerializer.serializeCall(
+            self._pyroConnection.objectId, methodname, vargs, kwargs,
             compress=Pyro4.config.COMPRESSION)
         if compressed:
             flags |= MessageFactory.FLAGS_COMPRESSED
@@ -286,6 +290,8 @@ class Proxy(object):
             flags |= MessageFactory.FLAGS_ONEWAY
         with self.__pyroLock:
             self._pyroSeq=(self._pyroSeq+1)&0xffff
+            if Pyro4.config.LOGWIRE:
+                log.debug("proxy wiredata sending: msgtype=%d flags=0x%x seq=%d data=%r" % (MessageFactory.MSG_INVOKE, flags, self._pyroSeq, data))
             data=MessageFactory.createMessage(MessageFactory.MSG_INVOKE, data, flags, self._pyroSeq)
             try:
                 self._pyroConnection.send(data)
@@ -293,9 +299,11 @@ class Proxy(object):
                 if flags & MessageFactory.FLAGS_ONEWAY:
                     return None    # oneway call, no response data
                 else:
-                    msgType, flags, seq, data = MessageFactory.getMessage(self._pyroConnection, MessageFactory.MSG_RESULT)
+                    msgType, flags, seq, data = MessageFactory.getMessage(self._pyroConnection, [MessageFactory.MSG_RESULT])
+                    if Pyro4.config.LOGWIRE:
+                        log.debug("proxy wiredata received: msgtype=%d flags=0x%x seq=%d data=%r" % (msgType, flags, seq, data) )
                     self.__pyroCheckSequence(seq)
-                    data=self._pyroSerializer.deserialize(data, compressed=flags & MessageFactory.FLAGS_COMPRESSED)
+                    data=self._pyroSerializer.deserializeData(data, compressed=flags & MessageFactory.FLAGS_COMPRESSED)
                     if flags & MessageFactory.FLAGS_EXCEPTION:
                         if sys.platform=="cli":
                             util.fixIronPythonExceptionForPickle(data, False)
@@ -338,7 +346,7 @@ class Proxy(object):
                         return False    # already connected
                     sock=socketutil.createSocket(connect=connect_location, reuseaddr=Pyro4.config.SOCK_REUSE, timeout=self.__pyroTimeout)
                     conn=socketutil.SocketConnection(sock, uri.object)
-                    # Do handshake. For now, no need to send anything.
+                    # Do handshake. For now, no need to send anything. (message type CONNECT is not yet used)
                     msgType, flags, seq, data = MessageFactory.getMessage(conn, None)
                     # any trailing data (dataLen>0) is an error message, if any
                 except Exception:
@@ -355,9 +363,9 @@ class Proxy(object):
                     if msgType==MessageFactory.MSG_CONNECTFAIL:
                         error="connection rejected"
                         if data:
-                            if sys.version_info>=(3,0):
-                                data=str(data,"utf-8")
-                            error+=", reason: "+data
+                            if sys.version_info>=(3, 0):
+                                data=str(data, "utf-8")
+                            error+=", reason: " + data
                         conn.close()
                         log.error(error)
                         raise errors.CommunicationError(error)
@@ -457,7 +465,7 @@ class _BatchProxyAdapter(object):
             if not oneway:
                 return self.__resultsgenerator(results)
 
-    def _pyroInvoke(self,name,args,kwargs):
+    def _pyroInvoke(self, name, args, kwargs):
         # ignore all parameters, we just need to execute the batch
         results=self.__proxy._pyroInvokeBatch(self.__calls)
         self.__calls=[]   # clear for re-use
@@ -527,25 +535,20 @@ class MessageFactory(object):
     MSG_CONNECTFAIL = 3
     MSG_INVOKE = 4
     MSG_RESULT = 5
+    MSG_PING = 6
     FLAGS_EXCEPTION = 1<<0
     FLAGS_COMPRESSED = 1<<1
     FLAGS_ONEWAY = 1<<2
     FLAGS_HMAC = 1<<3
     FLAGS_BATCH = 1<<4
     MAGIC = 0x34E9
-    if sys.version_info>=(3,0):
-        empty_bytes = bytes([])
-        pyro_tag = bytes("PYRO", "ASCII")
-        empty_hmac = bytes(hashlib.sha1().digest_size)
-    else:
-        empty_bytes = ""
-        pyro_tag = "PYRO"
-        empty_hmac = "\0"*hashlib.sha1().digest_size
+    pyro_tag = b"PYRO"
+    empty_hmac = b"\0"*hashlib.sha1().digest_size
 
     @classmethod
     def createMessage(cls, msgType, databytes, flags, seq):
         """creates a message containing a header followed by the given databytes"""
-        databytes=databytes or cls.empty_bytes
+        databytes=databytes or b""
         if 0 < Pyro4.config.MAX_MESSAGE_SIZE < len(databytes):
             raise errors.ProtocolError("max message size exceeded (%d where max=%d)" % (len(databytes), Pyro4.config.MAX_MESSAGE_SIZE))
         if Pyro4.config.HMAC_KEY:
@@ -570,7 +573,7 @@ class MessageFactory(object):
         return msgType, flags, seq, dataLen, datahmac
 
     @classmethod
-    def getMessage(cls, connection, requiredMsgType):
+    def getMessage(cls, connection, requiredMsgTypes):
         headerdata = connection.recv(cls.HEADERSIZE)
         msgType, flags, seq, datalen, datahmac = cls.parseMessageHeader(headerdata)
         if 0 < Pyro4.config.MAX_MESSAGE_SIZE < datalen:
@@ -578,8 +581,8 @@ class MessageFactory(object):
             log.error("connection "+str(connection)+": "+errorMsg)
             connection.close()   # close the socket because at this point we can't return the correct sequence number for returning an error message
             raise errors.ProtocolError(errorMsg)
-        if requiredMsgType is not None and msgType != requiredMsgType:
-            err="invalid msg type %d received" % msgType
+        if requiredMsgTypes and msgType not in requiredMsgTypes:
+            err = "invalid msg type %d received" % msgType
             log.error(err)
             raise errors.ProtocolError(err)
         databytes=connection.recv(datalen)
@@ -590,24 +593,19 @@ class MessageFactory(object):
         elif flags&MessageFactory.FLAGS_HMAC != local_hmac_set:
             # Message contains hmac and local HMAC_KEY not set, or vice versa. This is not allowed.
             err="hmac key config not symmetric"
-            log.warn(err)
+            log.warning(err)
             raise errors.SecurityError(err)
         return msgType, flags, seq, databytes
 
 
-def pyroObjectSerializer(self):
+def pyroObjectToAutoProxy(self):
     """reduce function that automatically replaces Pyro objects by a Proxy"""
-    daemon=getattr(self,"_pyroDaemon",None)
-    if daemon:
-        # only return a proxy if the object is a registered pyro object
-        return Pyro4.core.Proxy, (daemon.uriFor(self),)
-    else:
-        return self.__reduce__()
-
-
-def defaultObjectSerializer(self):
-    """reduce function that uses the default implementation"""
-    return self.__reduce__()
+    if Pyro4.config.AUTOPROXY:
+        daemon = getattr(self, "_pyroDaemon", None)
+        if daemon:
+            # only return a proxy if the object is a registered pyro object
+            return Pyro4.core.Proxy(daemon.uriFor(self))
+    return self
 
 
 class DaemonObject(object):
@@ -647,11 +645,7 @@ class Daemon(object):
             # choose the 'best' multiplexing implementation
             if os.name=="java":
                 raise NotImplementedError("select or poll-based server is not supported for jython, use thread server instead")
-            import select
-            if hasattr(select,"poll"):
-                self.transportServer=SocketServer_Poll()
-            else:
-                self.transportServer=SocketServer_Select()
+            self.transportServer = SocketServer_Poll() if socketutil.hasPoll else SocketServer_Select()
         else:
             raise errors.PyroError("invalid server type '%s'" % Pyro4.config.SERVERTYPE)
         self.transportServer.init(self, host, port, unixsocket)
@@ -666,7 +660,7 @@ class Daemon(object):
         self.natLocationStr = "%s:%d" % (nathost, natport_for_loc) if nathost else None
         if self.natLocationStr:
             log.debug("NAT address is %s", self.natLocationStr)
-        self.serializer=util.Serializer()
+        self.serializer=util.get_serializer()
         pyroObject=DaemonObject(self)
         pyroObject._pyroId=constants.DAEMON_NAME
         #: Dictionary from Pyro object id to the actual Pyro object registered by this id
@@ -748,10 +742,7 @@ class Daemon(object):
         # We need a minimal amount of data or the socket will remain blocked
         # on some systems... (messages smaller than 40 bytes)
         # Return True for successful handshake, False if something was wrong.
-        data="ok"
-        if sys.version_info>=(3,0):
-            data=bytes(data,"utf-8")
-        msg=MessageFactory.createMessage(MessageFactory.MSG_CONNECTOK, data, 0, 1)
+        msg=MessageFactory.createMessage(MessageFactory.MSG_CONNECTOK, b"ok", 0, 1)
         conn.send(msg)
         return True
 
@@ -766,9 +757,15 @@ class Daemon(object):
         wasBatched=False
         isCallback=False
         try:
-            msgType, flags, seq, data = MessageFactory.getMessage(conn, MessageFactory.MSG_INVOKE)
-            objId, method, vargs, kwargs=self.serializer.deserialize(
-                                           data, compressed=flags & MessageFactory.FLAGS_COMPRESSED)
+            msgType, flags, seq, data = MessageFactory.getMessage(conn, [MessageFactory.MSG_INVOKE, MessageFactory.MSG_PING])
+            if Pyro4.config.LOGWIRE:
+                log.debug("daemon wiredata received: msgtype=%d flags=0x%x seq=%d data=%r" % (msgType, flags, seq, data) )
+            if msgType == MessageFactory.MSG_PING:
+                # return same seq, but ignore any data (it's a ping, not an echo)
+                msg = MessageFactory.createMessage(MessageFactory.MSG_PING, None, 0, seq)
+                conn.send(msg)
+                return
+            objId, method, vargs, kwargs = self.serializer.deserializeCall(data, compressed=flags & MessageFactory.FLAGS_COMPRESSED)
             del data  # invite GC to collect the object, don't wait for out-of-scope
             obj=self.objectsById.get(objId)
             if obj is not None:
@@ -778,12 +775,12 @@ class Daemon(object):
                 if flags & MessageFactory.FLAGS_BATCH:
                     # batched method calls, loop over them all and collect all results
                     data=[]
-                    for method,vargs,kwargs in vargs:
+                    for method, vargs, kwargs in vargs:
                         method=util.resolveDottedAttribute(obj, method, Pyro4.config.DOTTEDNAMES)
                         try:
                             result=method(*vargs, **kwargs)   # this is the actual method call to the Pyro object
                         except Exception:
-                            xt,xv=sys.exc_info()[0:2]
+                            xt, xv = sys.exc_info()[0:2]
                             log.debug("Exception occurred while handling batched request: %s", xv)
                             xv._pyroTraceback=util.formatTraceback(detailed=Pyro4.config.DETAILED_TRACEBACK)
                             if sys.platform=="cli":
@@ -810,17 +807,19 @@ class Daemon(object):
             if flags & MessageFactory.FLAGS_ONEWAY:
                 return   # oneway call, don't send a response
             else:
-                data, compressed=self.serializer.serialize(data, compress=Pyro4.config.COMPRESSION)
+                data, compressed=self.serializer.serializeData(data, compress=Pyro4.config.COMPRESSION)
                 flags=0
                 if compressed:
                     flags |= MessageFactory.FLAGS_COMPRESSED
                 if wasBatched:
                     flags |= MessageFactory.FLAGS_BATCH
+                if Pyro4.config.LOGWIRE:
+                    log.debug("daemon wiredata sending: msgtype=%d flags=0x%x seq=%d data=%r" % (MessageFactory.MSG_RESULT, flags, seq, data))
                 msg=MessageFactory.createMessage(MessageFactory.MSG_RESULT, data, flags, seq)
                 del data
                 conn.send(msg)
         except Exception:
-            xt,xv=sys.exc_info()[0:2]
+            xt, xv = sys.exc_info()[0:2]
             if xt is not errors.ConnectionClosedError:
                 log.debug("Exception occurred while handling request: %r", xv)
                 if not flags & MessageFactory.FLAGS_ONEWAY:
@@ -836,7 +835,7 @@ class Daemon(object):
         if sys.platform=="cli":
             util.fixIronPythonExceptionForPickle(exc_value, True)  # piggyback attributes
         try:
-            data, _=self.serializer.serialize(exc_value)
+            data, compressed =self.serializer.serializeData(exc_value)
         except:
             # the exception object couldn't be serialized, use a generic PyroError instead
             xt, xv, tb = sys.exc_info()
@@ -845,8 +844,11 @@ class Daemon(object):
             exc_value._pyroTraceback=tbinfo
             if sys.platform=="cli":
                 util.fixIronPythonExceptionForPickle(exc_value, True)  # piggyback attributes
-            data, _=self.serializer.serialize(exc_value)
-        msg=MessageFactory.createMessage(MessageFactory.MSG_RESULT, data, MessageFactory.FLAGS_EXCEPTION, seq)
+            data, compressed =self.serializer.serializeData(exc_value)
+        flags = MessageFactory.FLAGS_EXCEPTION
+        if compressed:
+            flags |= MessageFactory.FLAGS_COMPRESSED
+        msg=MessageFactory.createMessage(MessageFactory.MSG_RESULT, data, flags, seq)
         del data
         connection.send(msg)
 
@@ -870,10 +872,7 @@ class Daemon(object):
         obj._pyroDaemon=self
         if Pyro4.config.AUTOPROXY:
             # register a custom serializer for the type to automatically return proxies
-            try:
-                copyreg.pickle(type(obj),pyroObjectSerializer)
-            except TypeError:
-                pass
+            self.serializer.register_type_replacement(type(obj), pyroObjectToAutoProxy)
         # register the object in the mapping
         self.objectsById[obj._pyroId]=obj
         return self.uriFor(objectId)
@@ -899,11 +898,8 @@ class Daemon(object):
             if objectOrId is not None:
                 del objectOrId._pyroId
                 del objectOrId._pyroDaemon
-                # Don't remove the custom type serializer (copyreg.pickle) because there
-                # may be other registered objects of the same type still depending on it.
-                # Also, it would require an inefficient linear search through the registered
-                # objects map to scan for types. Finally, the copyreg module doesn't seem
-                # to be designed with cleanup in mind (it has no explicit unregister function)
+                # Don't remove the custom type serializer because there may be
+                # other registered objects of the same type still depending on it.
 
     def uriFor(self, objectOrId=None, nat=True):
         """
@@ -934,7 +930,7 @@ class Daemon(object):
 
     def __repr__(self):
         return "<%s.%s at 0x%x, %s, %d objects>" % (self.__class__.__module__, self.__class__.__name__,
-            id(self), self.locationStr, len(self.objectsById))
+               id(self), self.locationStr, len(self.objectsById))
 
     def __enter__(self):
         if not self.transportServer:
@@ -947,6 +943,9 @@ class Daemon(object):
     def __getstate__(self):
         return {}   # a little hack to make it possible to serialize Pyro objects.
 
+    def __getstate_for_dict__(self):
+        return self.__getstate__()
+
 
 # decorators
 
@@ -960,4 +959,29 @@ def callback(object):
     return object
 
 
-# backward compatibility
+try:
+    import serpent
+    def pyro_class_serpent_serializer(obj, serializer, stream, level):
+        # Override the default way that a Pyro URI/proxy/daemon is serialized.
+        # Because it defines a __getstate__ it would otherwise just become a tuple,
+        # and not be deserialized as a class.
+        d = Pyro4.util.SerializerBase.class_to_dict(obj)
+        serializer.ser_builtins_dict(d, stream, level)
+    serpent.register_class(URI, pyro_class_serpent_serializer)
+    serpent.register_class(Proxy, pyro_class_serpent_serializer)
+    serpent.register_class(Daemon, pyro_class_serpent_serializer)
+    serpent.register_class(futures._ExceptionWrapper, pyro_class_serpent_serializer)
+except ImportError:
+    pass
+
+
+def serialize_core_object_to_dict(obj):
+    return {
+        "__class__": "Pyro4.core." + obj.__class__.__name__,
+        "state": obj.__getstate_for_dict__()
+    }
+
+Pyro4.util.SerializerBase.register_class_to_dict(URI, serialize_core_object_to_dict)
+Pyro4.util.SerializerBase.register_class_to_dict(Proxy, serialize_core_object_to_dict)
+Pyro4.util.SerializerBase.register_class_to_dict(Daemon, serialize_core_object_to_dict)
+Pyro4.util.SerializerBase.register_class_to_dict(futures._ExceptionWrapper, futures._ExceptionWrapper.__serialized_dict__)
