@@ -139,10 +139,7 @@ class NameServerDaemon(core.Daemon):
 
 
 class BroadcastServer(object):
-    if sys.version_info>=(3, 0):
-        REQUEST_NSURI=bytes("GET_NSURI", "ASCII")
-    else:
-        REQUEST_NSURI="GET_NSURI"
+    REQUEST_NSURI = "GET_NSURI" if sys.platform=="cli" else b"GET_NSURI"
 
     def __init__(self, nsUri, bchost=None, bcport=None):
         self.nsUri=nsUri
@@ -168,6 +165,10 @@ class BroadcastServer(object):
     def close(self):
         log.debug("ns broadcast server closing")
         self.running=False
+        try:
+            self.sock.shutdown(socket.SHUT_RDWR)
+        except (OSError, socket.error):
+            pass
         self.sock.close()
 
     def getPort(self):
@@ -202,9 +203,7 @@ class BroadcastServer(object):
                     except socket.error:
                         pass
                 log.debug("responding to broadcast request from %s: interface %s", addr[0], responsedata.host)
-                responsedata=str(responsedata)
-                if sys.version_info>=(3, 0):
-                    responsedata=bytes(responsedata, "iso-8859-1")
+                responsedata = str(responsedata).encode("iso-8859-1")
                 self.sock.sendto(responsedata, 0, addr)
         except socket.error:
             pass
@@ -243,6 +242,7 @@ def startNSloop(host=None, port=None, enableBroadcast=True, bchost=None, bcport=
         print("external URI = %s" % nsUri)
     else:
         print("URI = %s" % nsUri)
+    log.info("using serializer: %s" % Pyro4.config.SERIALIZER)   # only clients using the same serializer can connect
     try:
         daemon.requestLoop()
     finally:
@@ -303,13 +303,21 @@ def locateNS(host=None, port=None):
                             if err not in Pyro4.socketutil.ERRNO_EADDRINUSE:     # and jython likes to throw thses...
                                 raise
                 data, _=sock.recvfrom(100)
+                try:
+                    sock.shutdown(socket.SHUT_RDWR)
+                except (OSError, socket.error):
+                    pass
                 sock.close()
-                if sys.version_info>=(3,0):
+                if sys.version_info>=(3, 0):
                     data=data.decode("iso-8859-1")
                 log.debug("located NS: %s", data)
                 return core.Proxy(data)
             except socket.timeout:
                 continue
+        try:
+            sock.shutdown(socket.SHUT_RDWR)
+        except (OSError, socket.error):
+            pass
         sock.close()
         log.debug("broadcast locate failed, try direct connection on NS_HOST")
         # broadcast failed, try PYRO directly on specific host
@@ -358,7 +366,7 @@ def main(args):
     parser=OptionParser()
     parser.add_option("-n", "--host", dest="host", help="hostname to bind server on")
     parser.add_option("-p", "--port", dest="port", type="int", help="port to bind server on (0=random)")
-    parser.add_option("-u","--unixsocket", help="Unix domain socket name to bind server on")
+    parser.add_option("-u", "--unixsocket", help="Unix domain socket name to bind server on")
     parser.add_option("", "--bchost", dest="bchost", help="hostname to bind broadcast server on (default is \"\")")
     parser.add_option("", "--bcport", dest="bcport", type="int",
                       help="port to bind broadcast server on (0=random)")
