@@ -27,26 +27,30 @@ class ClientConnectionJob(object):
 
     def __call__(self):
         if self.handleConnection():
-            while True:
-                try:
-                    self.daemon.handleRequest(self.csock)
-                except (socket.error, errors.ConnectionClosedError):
-                    # client went away.
-                    log.debug("disconnected %s", self.caddr)
-                    break
-                except errors.SecurityError:
-                    log.debug("security error on client %s", self.caddr)
-                    break
-            self.csock.close()
+            try:
+                while True:
+                    try:
+                        self.daemon.handleRequest(self.csock)
+                    except (socket.error, errors.ConnectionClosedError):
+                        # client went away.
+                        log.debug("disconnected %s", self.caddr)
+                        break
+                    except errors.SecurityError:
+                        log.debug("security error on client %s", self.caddr)
+                        break
+                    # other errors simply crash the client worker thread (and close its connection)
+            finally:
+                self.csock.close()
 
     def handleConnection(self):
         # connection handshake
         try:
             if self.daemon._handshake(self.csock):
                 return True
-        except (socket.error, errors.PyroError):
-            x=sys.exc_info()[1]
-            log.warn("error during connect: %s", x)
+        except:
+            ex_t, ex_v, ex_tb = sys.exc_info()
+            tb = Pyro4.util.formatTraceback(ex_t, ex_v, ex_tb)
+            log.warning("error during connect/handshake: %s; %s", ex_v, "\n".join(tb))
             self.csock.close()
         return False
 
@@ -55,7 +59,7 @@ class ClientConnectionJob(object):
         try:
             self.csock.sock.shutdown(socket.SHUT_RDWR)
             self.csock.sock.setblocking(False)
-        except socket.error:
+        except (OSError, socket.error):
             pass
         if hasattr(socket, "SO_RCVTIMEO"):
             # setting a recv timeout seems to break the blocking call to recv() on some systems
@@ -78,13 +82,16 @@ class SocketServer_Threadpool(object):
         self._socketaddr=self.sock.getsockname()
         if self._socketaddr[0].startswith("127."):
             if host is None or host.lower()!="localhost" and not host.startswith("127."):
-                log.warn("weird DNS setup: %s resolves to localhost (127.x.x.x)", host)
+                log.warning("weird DNS setup: %s resolves to localhost (127.x.x.x)", host)
         if unixsocket:
             self.locationStr="./u:"+unixsocket
         else:
             host=host or self._socketaddr[0]
             port=port or self._socketaddr[1]
-            self.locationStr="%s:%d" % (host, port)
+            if ":" in host:  # ipv6
+                self.locationStr="[%s]:%d" % (host, port)
+            else:
+                self.locationStr="%s:%d" % (host, port)
         self.jobqueue = Pyro4.tpjobqueue.ThreadPooledJobQueue()
         log.info("%d workers started", self.jobqueue.workercount)
 
@@ -170,11 +177,11 @@ def interruptSocket(address):
     """bit of a hack to trigger a blocking server to get out of the loop, useful at clean shutdowns"""
     try:
         sock=socketutil.createSocket(connect=address, keepalive=False, timeout=None)
-        if sys.version_info<(3, 0):
-            sock.send("!"*16)
-        else:
-            sock.send(bytes([1]*16))
+        socketutil.triggerSocket(sock)
+        try:
+            sock.shutdown(socket.SHUT_RDWR)
+        except (OSError, socket.error):
+            pass
         sock.close()
     except socket.error:
         pass
-
