@@ -5,7 +5,6 @@ Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
 import socket, os, errno, time, sys
-import re
 from Pyro4.errors import ConnectionClosedError, TimeoutError, CommunicationError
 import Pyro4
 
@@ -82,7 +81,7 @@ def getIpAddress(hostname, workaround127=False, ipVersion=None):
             family=socket.AF_UNSPEC
         else:
             raise ValueError("unknown value for argument ipVersion.")
-        ip=socket.getaddrinfo(hostname or socket.gethostname(), None, family, socket.SOCK_STREAM, socket.SOL_TCP)[0][4][0]
+        ip=socket.getaddrinfo(hostname or socket.gethostname(), 80, family, socket.SOCK_STREAM, socket.SOL_TCP)[0][4][0]
         if workaround127 and (ip.startswith("127.") or ip=="0.0.0.0"):
             ip=getInterfaceAddress("4.2.2.2")
         return ip
@@ -105,7 +104,7 @@ def getIpAddress(hostname, workaround127=False, ipVersion=None):
 
 
 def getInterfaceAddress(ip_address):
-    """tries to find the ip address of the interface that connects to a given host"""
+    """tries to find the ip address of the interface that connects to the given host's address"""
     family = socket.AF_INET if getIpVersion(ip_address)==4 else socket.AF_INET6
     sock = socket.socket(family, socket.SOCK_DGRAM)
     sock.connect((ip_address, 53))   # 53=dns
@@ -124,12 +123,6 @@ def __nextRetrydelay(delay):
     return delay+0.1
 
 
-if sys.version_info<(3, 0):
-    EMPTY_BYTES=""
-else:
-    EMPTY_BYTES=bytes([])
-
-
 def receiveData(sock, size):
     """Retrieve a given number of bytes from a socket.
     It is expected the socket is able to supply that number of bytes.
@@ -141,6 +134,9 @@ def receiveData(sock, size):
         retrydelay=0.0
         msglen=0
         chunks=[]
+        EMPTY_BYTES = b""
+        if sys.platform=="cli":
+            EMPTY_BYTES = ""
         if USE_MSG_WAITALL:
             # waitall is very convenient and if a socket error occurs,
             # we can assume the receive has failed. No need for a loop,
@@ -176,7 +172,7 @@ def receiveData(sock, size):
                         break
                     chunks.append(chunk)
                     msglen+=len(chunk)
-                data=EMPTY_BYTES.join(chunks)
+                data = EMPTY_BYTES.join(chunks)
                 del chunks
                 if len(data)!=size:
                     err=ConnectionClosedError("receiving: not enough data")
@@ -440,6 +436,10 @@ class SocketConnection(object):
         return receiveData(self.sock, size)
 
     def close(self):
+        try:
+            self.sock.shutdown(socket.SHUT_RDWR)
+        except (OSError, socket.error):
+            pass
         self.sock.close()
 
     def fileno(self):
@@ -468,24 +468,22 @@ def findProbablyUnusedPort(family=socket.AF_INET, socktype=socket.SOCK_STREAM):
 def bindOnUnusedPort(sock, host='localhost'):
     """Bind the socket to a free port and return the port number.
     This code is based on the code in the stdlib's test.test_support module."""
-    socketfamily = getattr(sock, "family", socket.AF_INET)   # workaround for jython bug http://bugs.jython.org/issue1803
-    sockettype = getattr(sock, "type", socket.SOCK_STREAM)   # workaround for jython bug http://bugs.jython.org/issue1804
-    if os.name!="java" and socketfamily in(socket.AF_INET, socket.AF_INET6) and sockettype == socket.SOCK_STREAM:
+    if os.name!="java" and sock.family in(socket.AF_INET, socket.AF_INET6) and sock.type == socket.SOCK_STREAM:
         if hasattr(socket, "SO_EXCLUSIVEADDRUSE"):
             # even though Jython has this socket option, it doesn't support it. Hence the check in the if statement above.
             sock.setsockopt(socket.SOL_SOCKET, socket.SO_EXCLUSIVEADDRUSE, 1)
-    if socketfamily == socket.AF_INET:
+    if sock.family == socket.AF_INET:
         if host == 'localhost':
             sock.bind(('127.0.0.1', 0))
         else:
             sock.bind((host, 0))
-    elif socketfamily == socket.AF_INET6:
+    elif sock.family == socket.AF_INET6:
         if host == 'localhost':
             sock.bind(('::1', 0, 0, 0))
         else:
             sock.bind((host, 0, 0, 0))
     else:
-        raise CommunicationError( "unsupported socket family: " + socketfamily )
+        raise CommunicationError( "unsupported socket family: " + sock.family )
     if os.name=="java":
         try:
             sock.listen(100)  # otherwise jython always just returns 0 for the port
@@ -493,3 +491,20 @@ def bindOnUnusedPort(sock, host='localhost'):
             pass  # jython sometimes throws errors here
     port = sock.getsockname()[1]
     return port
+
+
+"""is select() available?"""
+hasSelect = select and hasattr(select, "select")
+
+"""is poll() available?"""
+hasPoll = select and hasattr(select, "poll")
+
+def triggerSocket(sock):
+    """send a small data packet over the socket, to trigger it"""
+    try:
+        data = b"!"*16
+        if sys.platform=="cli":
+            data = "!"*16
+        sock.send(data)
+    except socket.error:
+        pass
