@@ -1,6 +1,7 @@
 """
 Pyro FLAME:  Foreign Location Automatic Module Exposer.
 Easy but potentially very dangerous way of exposing remote modules and builtins.
+Flame requires the pickle serializer to be used.
 
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
@@ -12,6 +13,7 @@ import code
 import Pyro4.core
 import Pyro4.util
 import Pyro4.constants
+import Pyro4.errors
 try:
     import importlib
 except ImportError:
@@ -52,6 +54,8 @@ def fixExecSourceNewlines(source):
         # it needs unix type newlines and a trailing newline to work correctly.
         source = source.replace("\r\n", "\n")
         source = source.rstrip() + "\n"
+    # remove trailing whitespace that might cause IndentationErrors
+    source = source.rstrip()
     return source
 
 
@@ -178,6 +182,10 @@ class Flame(object):
     Usually created by using :py:meth:`Pyro4.core.Daemon.startFlame`.
     Be *very* cautious before starting this: it allows the clients full access to everything on your system.
     """
+    def __init__(self):
+        if Pyro4.config.SERIALIZER != "pickle":
+            raise RuntimeError("flame requires the pickle serializer")
+
     def module(self, name):
         """import a module on the server given by the module name and returns a proxy to it"""
         if importlib:
@@ -275,7 +283,10 @@ def start(daemon):
     Create and register a Flame server in the given daemon.
     Be *very* cautious before starting this: it allows the clients full access to everything on your system.
     """
-    return daemon.register(Flame(), Pyro4.constants.FLAME_NAME)
+    if Pyro4.config.FLAME_ENABLED:
+        return daemon.register(Flame(), Pyro4.constants.FLAME_NAME)
+    else:
+        raise Pyro4.errors.SecurityError("Flame is disabled in the server configuration")
 
 
 def connect(location):
