@@ -6,15 +6,12 @@ Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 
 from __future__ import with_statement
 import os, time, socket
-import unittest
 import Pyro4.core
 import Pyro4.constants
 import Pyro4.socketutil
 from Pyro4.errors import DaemonError,PyroError
 from testsupport import *
 
-Pyro4.config.reset(useenvironment=False)
-
 
 class MyObj(object):
     def __init__(self, arg):
@@ -29,9 +26,9 @@ class DaemonTests(unittest.TestCase):
 
     def setUp(self):
         Pyro4.config.POLLTIMEOUT=0.1
-        Pyro4.config.HMAC_KEY=tobytes("testsuite")
+        Pyro4.config.HMAC_KEY = b"testsuite"
     def tearDown(self):
-        Pyro4.config.HMAC_KEY=None
+        Pyro4.config.HMAC_KEY = None
         
     def testDaemon(self):
         with Pyro4.core.Daemon(port=0) as d:
