@@ -5,7 +5,6 @@ Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
 from __future__ import with_statement
-import unittest
 import time
 import Pyro4.core
 import Pyro4.naming
@@ -29,11 +28,11 @@ class NSLoopThread(threadutil.Thread):
 
 class BCSetupTests(unittest.TestCase):
     def setUp(self):
-        Pyro4.config.HMAC_KEY=tobytes("testsuite")
+        Pyro4.config.HMAC_KEY=b"testsuite"
     def tearDown(self):
         Pyro4.config.HMAC_KEY=None
     def testBCstart(self):
-        myIpAddress=Pyro4.socketutil.getMyIpAddress(workaround127=True)
+        myIpAddress=Pyro4.socketutil.getIpAddress("", workaround127=True)
         nsUri, nameserver, bcserver = Pyro4.naming.startNS(host=myIpAddress, port=0, bcport=0, enableBroadcast=False)
         self.assertTrue(bcserver is None)
         nameserver.close()
@@ -45,19 +44,11 @@ class BCSetupTests(unittest.TestCase):
         bcserver.close()
 
 
-#This test is disabled because otherwise the suite won't succeed if you have a name server running somewhere:
-#class NameServerNotRunningTests(unittest.TestCase):
-#    def testLocate(self):
-#        Pyro4.config.HMAC_KEY=tobytes("testsuite")
-#        self.assertRaises(NamingError, Pyro4.naming.locateNS)
-#        Pyro4.config.HMAC_KEY=None
-
-
 class NameServerTests(unittest.TestCase):
     def setUp(self):
         Pyro4.config.POLLTIMEOUT=0.1
-        Pyro4.config.HMAC_KEY=tobytes("testsuite")
-        myIpAddress=Pyro4.socketutil.getMyIpAddress(workaround127=True)
+        Pyro4.config.HMAC_KEY=b"testsuite"
+        myIpAddress=Pyro4.socketutil.getIpAddress("", workaround127=True)
         self.nsUri, self.nameserver, self.bcserver = Pyro4.naming.startNS(host=myIpAddress, port=0, bcport=0)
         self.assertTrue(self.bcserver is not None,"expected a BC server to be running")
         self.bcserver.runInThread()
@@ -167,7 +158,8 @@ class NameServerTests(unittest.TestCase):
         self.assertEqual("PYRO:12345@host.com:4444", str(resolved1))
         
         ns=Pyro4.naming.locateNS(self.nsUri.host, self.nsUri.port)
-        uri=Pyro4.naming.resolve("PYRONAME:"+Pyro4.constants.NAMESERVER_NAME+"@"+self.nsUri.host+":"+str(self.nsUri.port))
+        host="["+self.nsUri.host+"]" if ":" in self.nsUri.host else self.nsUri.host
+        uri=Pyro4.naming.resolve("PYRONAME:"+Pyro4.constants.NAMESERVER_NAME+"@"+host+":"+str(self.nsUri.port))
         self.assertEqual("PYRO",uri.protocol)
         self.assertEqual(self.nsUri.host,uri.host)
         self.assertEqual(Pyro4.constants.NAMESERVER_NAME,uri.object)
@@ -181,7 +173,7 @@ class NameServerTests(unittest.TestCase):
         self.assertEqual("PYRO",uri.protocol)
 
         # test some errors
-        self.assertRaises(NamingError, Pyro4.naming.resolve, "PYRONAME:unknown_object@"+self.nsUri.host)
+        self.assertRaises(NamingError, Pyro4.naming.resolve, "PYRONAME:unknown_object@"+host)
         self.assertRaises(TypeError, Pyro4.naming.resolve, 999)  #wrong arg type
 
 
@@ -196,7 +188,7 @@ class NameServerTests(unittest.TestCase):
 class NameServerTests0000(unittest.TestCase):
     def setUp(self):
         Pyro4.config.POLLTIMEOUT=0.1
-        Pyro4.config.HMAC_KEY=tobytes("testsuite")
+        Pyro4.config.HMAC_KEY=b"testsuite"
         self.nsUri, self.nameserver, self.bcserver = Pyro4.naming.startNS(host="", port=0, bcport=0)
         self.assertEqual("0.0.0.0", self.nsUri.host, "for hostname \"\" the resulting ip must be 0.0.0.0")
         self.assertTrue(self.bcserver is not None,"expected a BC server to be running")
