@@ -8,8 +8,11 @@ Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 from __future__ import with_statement
 import sys
 import threading
+import pickle
 
-__all__=["tobytes", "unicode", "unichr", "StringIO", "next", "AtomicCounter"]
+
+__all__=["tobytes", "unicode", "unichr", "StringIO", "next", "AtomicCounter",
+        "NonserializableError", "MyThing2", "unittest" ]
 
 if sys.version_info<(3,0):
     from StringIO import StringIO
@@ -32,6 +35,13 @@ else:
     next=next
 
 
+if (sys.version_info >= (2, 7) and sys.version_info < (3, 0)) or \
+        (sys.version_info >= (3, 1)):
+    import unittest
+else:
+    import unittest2 as unittest
+
+
 class AtomicCounter(object):
     def __init__(self):
         self.lock = threading.Lock()
@@ -45,3 +55,14 @@ class AtomicCounter(object):
         with self.lock:
             return self.count
 
+
+class NonserializableError(Exception):
+    def __reduce__(self):
+        raise pickle.PicklingError("to make this error non-serializable")
+
+
+class MyThing2(object):
+    def __init__(self, name="?"):
+        self.name = name
+
+
